#'Computes Probabilistic Information of a Forecast Relative to a Threshold or a Quantile
#'
#'Compute probabilistic bins of a set of forecast years ('fcyr') relative to 
#'the forecast climatology over the whole period of anomalies, optionally excluding 
#'the selected forecast years ('fcyr') or the forecast year for which the 
#'probabilistic bins are being computed (see 'compPeriod').
#'
#'@param ano Array of anomalies from Ano().\cr
#'  Must be of dimension (nexp/nobs, nmemb, nsdates, nleadtime, nlat,  nlon)
#'@param fcyr Indices of the forecast years of the anomalies which to compute 
#'  the probabilistic bins for, or 'all' to compute the bins for all the 
#'  years.\cr
#'  E.g., c(1:5), c(1, 4), 4 or 'all'.
#'@param thr Values used as thresholds to bin the anomalies.
#'@param quantile If quantile is TRUE (default), the threshold ('thr') 
#'  are quantiles.\cr
#'  If quantile is FALSE the thresholds ('thr') introduced are the absolute 
#'  thresholds of the bins.
#'@param posdates Position of the dimension in \code{ano} that corresponds to 
#'  the start dates (default = 3).
#'@param posdim Position of the dimension in \code{ano} which will be combined 
#'  with 'posdates' to compute the quantiles (default = 2, ensemble members).
#'@param compPeriod Three options: 
#'  "Full period"/"Without fcyr"/"Cross-validation" (The probabilities are 
#'  computed with the terciles based on ano/ano with all 'fcyr's 
#'  removed/cross-validation). The default is "Full period".
#'
#'@return Array with probabilistic information and dimensions:\cr
#'  c(length('thr') + 1, length(fcyr), nmemb/nparam, nmod/nexp/nobs, 
#'  nltime, nlat, nlon)\cr
#'  The values along the first dimension take values 0 or 1 depending on which 
#'  of the 'thr'+1 cathegories the forecast/observation at the corresponding 
#'  grid point, time step, member and starting date belongs to.
#'
#'@keywords datagen
#'@examples
#'# See examples on Load() to understand the first lines in this example
#'  \dontrun{
#'data_path <- system.file('sample_data', package = 's2dverification')
#'expA <- list(name = 'experiment', path = file.path(data_path,
#'             'model/$EXP_NAME$/$STORE_FREQ$_mean/$VAR_NAME$_3hourly',
#'             '$VAR_NAME$_$START_DATE$.nc'))
#'obsX <- list(name = 'observation', path = file.path(data_path,
#'             '$OBS_NAME$/$STORE_FREQ$_mean/$VAR_NAME$',
#'             '$VAR_NAME$_$YEAR$$MONTH$.nc'))
#'
#'# Now we are ready to use Load().
#'startDates <- c('19851101', '19901101', '19951101', '20001101', '20051101')
#'sampleData <- Load('tos', list(expA), list(obsX), startDates,
#'                   output = 'lonlat', latmin = 27, latmax = 48, 
#'                   lonmin = -12, lonmax = 40)
#'  }
#'  \dontshow{
#'startDates <- c('19851101', '19901101', '19951101', '20001101', '20051101')
#'sampleData <- s2dverification:::.LoadSampleData('tos', c('experiment'),
#'                                                c('observation'), startDates,
#'                                                output = 'lonlat',
#'                                                latmin = 27, latmax = 48,
#'                                                lonmin = -12, lonmax = 40)
#'  }
#'clim <- Clim(sampleMap$mod, sampleMap$obs)
#'ano_exp <- Ano(sampleMap$mod, clim$clim_exp)
#'PB <- ProbBins(ano_exp, fcyr = 3, thr = c(1/3, 2/3), quantile = TRUE, posdates = 3,
#'               posdim = 2)
#'
#'@export

ProbBins_new <- function(data, len = 'all', thr, quantile, sdate_dim = 'sdate', 
                     memb_dim = 'member',
                     compPeriod = "Full period", ncores) {

  # Check inputs 
  ## dims
  if (is.null(memb_dim) && compPeriod != "Cross-validation") {
     dims <- sdate_dim
  } else {
     dims <- c(sdate_dim, memb_dim)
  }
  
  ## data
  if (is.null(data)) {
    stop("Parameter 'data' cannot be NULL.")
  }
  if (!is.numeric(data)) {
    stop("Parameter 'data' must be a numeric array.")
  }
  if (is.null(dim(data))) {  #is vector, turn into array
    data <- array(data, c(dim = length(data)))
    dims <- 'dim'
  }
  ## forecast year
  if (length(len) == 1) {
    if (len == 'all') {
      if (length(dim(data)) == 1) {
        len <- array(1:length(data), length(data))
        names(dim(len)) <- names(dim(data))
      } else { 
        len <- array(1:dim(data)[sdate_dim], dim(data)[sdate_dim])
      }
    }
  }
  if (compPeriod == "Cross-validation") {
    result <- lapply(len, function(x) {
      if (quantile) {
        thr <- Apply(list(ClimProjDiags::Subset(data,
                     along = sdate_dim, indices = -x)), target_dims = c(dims),
                     fun = function(x) {quantile(as.vector(x), probs = thr,
                     na.rm = TRUE, names = FALSE, type = 8)},
                     output_dims = 'bin', ncores = ncores)$output1
      }
      data <- ClimProjDiags::Subset(data, along = sdate_dim, indices = x)
       Apply(list(data, thr), target_dims = list(c(dims), 'bin'),
                      fun = .bin, ncores = ncores)$output1
     })
    dims <- c(sdate = length(len), dim(result[[1]])[-1])
    result <- unlist(result)
    dim(result) <- dims
  } else if (compPeriod == "Without fcyr") {
    if (quantile) {
      thr <- Apply(list(ClimProjDiags::Subset(data,
                   along = sdate_dim, indices = -len)), target_dims = c(dims),
                   fun = function(x) {quantile(as.vector(x), probs = thr,
                   na.rm = TRUE, names = FALSE, type = 8)},
                   output_dims = 'bin', ncores = ncores)$output1
    }
    data <- ClimProjDiags::Subset(data, along = sdate_dim, indices = len)
    result <- Apply(list(data, thr), target_dims = list(c(dims), 'bin'),
                 fun = .bin, ncores = ncores)$output1
  } else if (compPeriod == "Full period") {
   if (quantile) {
     thr <- Apply(list(data), target_dims = c(dims),
                  fun = function(x) {quantile(as.vector(x), probs = thr,
                  na.rm = TRUE, names = FALSE, type = 8)},
                  output_dims = 'bin', ncores = ncores)$output1
   }
   result <- Apply(list(data, thr), target_dims = list(c(dims), 'bin'),
                fun = .bin, ncores = ncores)$output1
  } else {
    stop("Parameter 'compPeriod' must be one of 'Full period', ",
         "'Without fcyr' or 'Cross-validation'.")
  }
  return(result)
}
  
.PBF <- function(data, thr, quantile, dims = c('posdate', 'member'),
                 ncores) {
  if (quantile) {
    thr <- Apply(list(data), target_dims = c(dims),
                 fun = function(x) {quantile(as.vector(x), probs = thr,
                 na.rm = TRUE, names = FALSE, type = 8)}, 
                 output_dims = 'bin', ncores = ncores)$output1
  }
  PBF <- Apply(list(data, thr), target_dims = list(c(dims), 'bin'),
               fun = .bin, ncores = ncores)$output1
}
# data <- array(1:15, c(x = 15))
# thres <- c(5,10)
.bin <- function(data, thres) {
  res <- 1 * (data <= thres[1])
  if (length(thres) > 1) {
    res <- c(res, unlist(lapply(2:length(thres), function (i) {
      return(1 * ((data > thres[i - 1]) & (data <= thres[i])))
    })))
  }
  res <- c(res, 1 * (data > thres[length(thres)])) 
  dim(res) <- c(dim(data), bin = length(thres) + 1)
  return(res)
}
